/*
* This file is part of Firestorm NIDS
* Copyright (c) 2001 Gianni Tedesco
*
*    Firestorm is free software; you can redistribute it and/or modify
*    it under the terms of the GNU General Public License as published by
*    the Free Software Foundation; either version 2 of the License, or
*    (at your option) any later version.
*
*    Firestorm is distributed in the hope that it will be useful,
*    but WITHOUT ANY WARRANTY; without even the implied warranty of
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*    GNU General Public License for more details.
*
*    You should have received a copy of the GNU General Public License
*    along with Firestorm; if not, write to the Free Software
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#include <s_globals.h>
#include <s_util.h>

#define PRINT_FD 1

void print_raw ( char *raw )
{
	CRITICAL_START(globals.glock);
	write(PRINT_FD, raw, strlen(raw));
	CRITICAL_STOP(globals.glock);
}

int print_out(char *format, ...)
{
	va_list va;
	char output[BUFFER_SIZE];
	char *p=(char *)&output;

	memset(p, 0, BUFFER_SIZE);

	if ( format == NULL ) return 0;

	va_start(va, format);
	vsnprintf(p, BUFFER_SIZE-1, format, va);
	va_end(va);
	CRITICAL_START(globals.glock);
	write(PRINT_FD, p, strlen(p));
	CRITICAL_STOP(globals.glock);
	return 1;
}

int file_out(int fd, char *format, ...)
{
	va_list va;
	char output[BUFFER_SIZE];
	char *p=(char *)&output;

	memset(p, 0, BUFFER_SIZE);

	if ( format == NULL ) return 0;

	va_start(va, format);
	vsnprintf(p, BUFFER_SIZE-1, format, va);
	va_end(va);
	write(fd, p, strlen(p));
	return 1;
}

void file_raw (int fd, char *raw )
{
	write(fd, raw, strlen(raw));
}

int atomic_inc_counter(p_atomic_counter c)
{
	CRITICAL_START(c->lock);
	c->counter++;
	CRITICAL_STOP(c->lock);
	return c->counter;
}

int atomic_dec_counter(p_atomic_counter c)
{
	CRITICAL_START(c->lock);
	c->counter--;
	CRITICAL_STOP(c->lock);
	return c->counter;
}
